@@ -290,24 +290,16 @@ def kwargs2PlotAxes2D(kw: dict) -> PlotAxes2D:
 class PlotAstroData(AstroFrame):
     """Make a figure from 2D/3D FITS files or 2D/3D arrays.
 
-    Basic rules --- For 3D data, a 1D velocity array or a FITS file
-    with a velocity axis must be given to set up channels in each page.
-    For 2D/3D data, the spatial center can be read from a FITS file
-    or manually given.
+    Basic rules ---
+    For 3D data, a 1D velocity array or a FITS file with a velocity axis must be given to set up channels in each page.
+    For 2D/3D data, the spatial center can be read from a FITS file or manually given.
     len(v)=1 (default) means to make a 2D figure.
     Spatial lengths are in the unit of arcsec, or au if dist (!= 1) is given.
     Angles are in the unit of degree.
-    For region, line, arrow, label, and marker,
-    a single input can be treated without a list, e.g., anglelist=60,
-    as well as anglelist=[60].
-    Each element of poslist supposes a text coordinate
-    like '01h23m45.6s 01d23m45.6s' or a list of relative x and y
-    like [0.2, 0.3] (0 is left or bottom, 1 is right or top).
-    Parameters for original methods in matplotlib.axes.Axes can be
-    used as kwargs; see the default _kw for reference.
-    Position-velocity diagrams (pv=True) does not yet suppot region, line,
-    arrow, and segment because the units of abscissa and ordinate
-    are different.
+    For region, line, arrow, label, and marker, a single input can be treated without a list, e.g., anglelist=60, as well as anglelist=[60].
+    Each element of poslist supposes a text coordinate like '01h23m45.6s 01d23m45.6s' or a list of relative x and y like [0.2, 0.3] (0 is left or bottom, 1 is right or top).
+    Parameters for original methods in matplotlib.axes.Axes can be used as kwargs; see the default _kw for reference.
+    Position-velocity diagrams (pv=True) does not yet suppot region, line, arrow, and segment because the units of abscissa and ordinate are different.
 
     kwargs is the arguments of AstroFrame to define plotting ranges.
 
@@ -356,12 +348,11 @@ class PlotAstroData(AstroFrame):
             else:
                 v = v[:k1]
         if self.pv or v is None or len(v) == 1:
-            nv = nrows = ncols = npages = nchan = nchanplot = 1
+            nv = nrows = ncols = npages = nchan = 1
         else:
             nv = len(v := v[::vskip])
             npages = int(np.ceil(nv / nrows / ncols))
             nchan = npages * nrows * ncols
-            nchanplot = len(v)
             v = np.r_[v, v[-1] + (np.arange(nchan-nv)+1) * (v[1] - v[0])]
             if type(channelnumber) is int:
                 nchan = npages = 1
@@ -379,15 +370,15 @@ class PlotAstroData(AstroFrame):
             fontsize = 18 if nchan == 1 else 12
         set_rcparams(fontsize=fontsize, nancolor=nancolor, dpi=dpi)
         ax = np.empty(nchan, dtype='object') if internalax else [ax]
+        if figsize is None:
+            sqrt_a = (self.ymax - self.ymin) / (self.xmax - self.xmin)
+            sqrt_a = np.sqrt(np.abs(sqrt_a))
+            if nchan == 1:
+                figsize = (7 / sqrt_a, 5 * sqrt_a)
+            else:
+                figsize = (ncols * 2 / sqrt_a, max(nrows*2, 3) * sqrt_a)
         for ch in range(nchan):
             n, i, j = ch2nij(ch)
-            if figsize is None:
-                sqrt_a = (self.ymax - self.ymin) / (self.xmax - self.xmin)
-                sqrt_a = np.sqrt(np.abs(sqrt_a))
-                if nchan == 1:
-                    figsize = (7 / sqrt_a, 5 * sqrt_a)
-                else:
-                    figsize = (ncols * 2 / sqrt_a, max(nrows*2, 3) * sqrt_a)
             if internalfig:
                 fig = plt.figure(n, figsize=figsize)
             sharex = ax[nij2ch(n, i - 1, j)] if i > 0 else None
@@ -397,8 +388,8 @@ class PlotAstroData(AstroFrame):
                                          sharex=sharex, sharey=sharey)
             if nchan > 1 or type(channelnumber) is int:
                 fig.subplots_adjust(hspace=0, wspace=0, right=0.87, top=0.87)
-                vellabel = v[ch] if channelnumber is None else v[channelnumber]
-                if ch < nchanplot:
+                if ch < nv:
+                    vellabel = v[ch] if channelnumber is None else v[channelnumber]
                     ax[ch].text(0.9 * self.rmax, 0.7 * self.rmax,
                                 rf'${vellabel:.{veldigit:d}f}$', color='black',
                                 backgroundcolor='white', zorder=20)
@@ -406,7 +397,7 @@ class PlotAstroData(AstroFrame):
         self.ax = ax
         self.rowcol = nrows * ncols
         self.npages = npages
-        self.allchan = np.arange(nchan if channelnumber is None else nv)
+        self.allchan = np.arange(nv)
         self.bottomleft = nij2ch(np.arange(npages), nrows - 1, 0)
         self.channelnumber = channelnumber
 
@@ -484,25 +475,66 @@ class PlotAstroData(AstroFrame):
     def add_beam(self,
                  beam: list[float | None, float | None, float | None] = [None, None, None],
                  beamcolor: str = 'gray',
-                 poslist: list[str | list[float, float]] | None = None) -> None:
+                 beampos: list[float, float] | None = None,
+                 dv: float | None = None, pvpa: float | None = None) -> None:
         """Use add_region().
 
         Args:
             beam (list, optional): [bmaj, bmin, bpa]. Defaults to [None, None, None].
             beamcolor (str, optional): matplotlib color. Defaults to 'gray'.
-            poslist (list, optional): text or relative. Defaults to None.
+            beampos (list, optional): relative position. Defaults to None.
+            dv (float, optional): velocity resolution for a PV diagram. Defaults to None.
+            pvpa (float, optional): position angle of the PV cut in the unit of degrees. Defaults to None.
         """
         if None in beam:
             print('No beam to plot.')
             return False
 
-        if poslist is None:
-            poslist = [max(0.35 * beam[0] / self.rmax, 0.1)] * 2
         include_chan = self.bottomleft if self.channelnumber is None else self.allchan
-        self.add_region('ellipse', poslist, *beam,
+        if self.pv:
+            if pvpa is None:
+                pvpa = beam[0]
+                print('pvpa is not specified. pvpa=bmaj is assumed.')
+            p = np.radians(beam[2] - pvpa)
+            b = 1 / np.hypot(np.cos(p) / beam[0], np.sin(p) / beam[1]) / self.rmax
+            b = np.array([b, dv])
+            if beampos is None:
+                beampos = [max(0.35 * b[0], 0.1), 0.1]
+            if self.swapxy:
+                beampos = np.transpose(beampos)
+                b = np.transpose(b)
+            b = np.append(b, 0)
+            patch = 'rectangle'
+        else:
+            if beampos is None:
+                beampos = [max(0.35 * beam[0] / self.rmax, 0.1)] * 2
+            b = beam
+            patch = 'ellipse'
+        self.add_region(patch=patch, poslist=beampos,
+                        majlist=b[0], minlist=b[1], palist=b[2],
                         include_chan=include_chan,
                         facecolor=beamcolor, edgecolor=None)
 
+    def _beam(self, show_beam: bool, beamcolor: str | list[str],
+              beampos: list[float, float] | None, d: AstroData) -> None:
+        """Internal wrapper of add_beam().
+
+        Args:
+            show_beam (bool): Whether to show the beam(s).
+            beamcolor (str | list[str]): Color name.
+            beampos (list): Relative position.
+            d (AstroData): For reading the velocity resolution, beam, and pvpa.
+        """
+        if show_beam:
+            dv = d.v[1] - d.v[0] if self.pv else None
+            c = type(beamcolor) is list
+            blist = d.beam if c or np.ndim(d.beam) == 2 else [d.beam]
+            bclist = beamcolor if c else [beamcolor]
+            bplist = beampos if c else [beampos]
+            palist = d.pvpa if c else [d.pvpa]
+            for b, bc, bp, pa in zip(blist, bclist, bplist, palist):
+                self.add_beam(beam=b, beamcolor=bc, beampos=bp, dv=dv, pvpa=pa)
+
     def add_marker(self, poslist: list[str | list[float, float]] = [],
                    include_chan: list[int] | None = None, **kwargs) -> None:
         """Use Axes.plot of matplotlib.
@@ -639,7 +671,7 @@ class PlotAstroData(AstroFrame):
                   cbformat: float = '%.1e', cbticks: list[float] | None = None,
                   cbticklabels: list[str] | None = None, cblocation: str = 'right',
                   show_beam: bool = True, beamcolor: str = 'gray',
-                  **kwargs) -> None:
+                  beampos: list[float, float] | None = None, **kwargs) -> None:
         """Use Axes.pcolormesh of matplotlib. kwargs must include the arguments of AstroData to specify the data to be plotted.
 
         Args:
@@ -655,6 +687,7 @@ class PlotAstroData(AstroFrame):
             cblocation (str, optional): 'left', 'top', 'left', 'right'. Only for 2D images. Defaults to 'right'.
             show_beam (bool, optional): Defaults to True.
             beamcolor (str, optional): Matplotlib color. Defaults to 'gray'.
+            beampos (list, optional): Relative position. Defaults to None.
         """
         _kw = {'cmap': 'cubehelix', 'alpha': 1, 'edgecolors': 'none', 'zorder': 1}
         _kw.update(kwargs)
@@ -715,13 +748,12 @@ class PlotAstroData(AstroFrame):
                     ticklin = 1 + (1 - stretchpower) * np.log(10) * t
                     ticklin = cmin_org * ticklin**(1 / (1 - stretchpower))
                 cb.set_ticklabels([f'{d:{cbformat[1:]}}' for d in ticklin])
-        if show_beam and not self.pv:
-            self.add_beam(beam, beamcolor)
+        self._beam(show_beam, beamcolor, beampos, d)
 
     def add_contour(self, xskip: int = 1, yskip: int = 1,
                     levels: list[float] = [-12, -6, -3, 3, 6, 12, 24, 48, 96, 192, 384],
                     show_beam: bool = True, beamcolor: str = 'gray',
-                    **kwargs) -> None:
+                    beampos: list[float, float] | None = None, **kwargs) -> None:
         """Use Axes.contour of matplotlib. kwargs must include the arguments of AstroData to specify the data to be plotted.
 
         Args:
@@ -729,6 +761,7 @@ class PlotAstroData(AstroFrame):
             levels (list, optional): Contour levels in the unit of sigma. Defaults to [-12,-6,-3,3,6,12,24,48,96,192,384].
             show_beam (bool, optional): Defaults to True.
             beamcolor (str, optional): Matplotlib color. Defaults to 'gray'.
+            beampos (list, optional): Relative position. Defaults to None.
         """
         _kw = {'colors': 'gray', 'linewidths': 1.0, 'zorder': 2}
         _kw.update(kwargs)
@@ -742,8 +775,7 @@ class PlotAstroData(AstroFrame):
             c = [c[self.channelnumber]]
         for axnow, cnow in zip(self.ax, c):
             axnow.contour(x, y, cnow, np.sort(levels) * sigma, **_kw)
-        if show_beam and not self.pv:
-            self.add_beam(beam, beamcolor)
+        self._beam(show_beam, beamcolor, beampos, d)
 
     def add_segment(self, ampfits: str = None, angfits: str = None,
                     Ufits: str = None, Qfits: str = None,
@@ -756,7 +788,7 @@ class PlotAstroData(AstroFrame):
                     rotation: float = 0.,
                     cutoff: float = 3.,
                     show_beam: bool = True, beamcolor: str = 'gray',
-                    **kwargs) -> None:
+                    beampos: list[float, float] | None = None, **kwargs) -> None:
         """Use Axes.quiver of matplotlib. kwargs must include the arguments of AstroData to specify the data to be plotted. fitsimage = [ampfits, angfits, Ufits, Qfits]. data = [amp, ang, stU, stQ].
 
         Args:
@@ -775,6 +807,7 @@ class PlotAstroData(AstroFrame):
             cutoff (float, optional): Used when amp and ang are calculated from Stokes U and Q. In the unit of sigma. Defaults to 3..
             show_beam (bool, optional): Defaults to True.
             beamcolor (str, optional): Matplotlib color. Defaults to 'gray'.
+            beampos (list, optional): Relative position. Defaults to None.
         """
         _kw = {'angles': 'xy', 'scale_units': 'xy', 'color': 'gray',
                'pivot': 'mid', 'headwidth': 0, 'headlength': 0,
@@ -809,8 +842,7 @@ class PlotAstroData(AstroFrame):
         _kw['scale'] = 1 if len(x) == 1 else 1. / np.abs(x[1] - x[0])
         for axnow, unow, vnow in zip(self.ax, U, V):
             axnow.quiver(x, y, unow, vnow, **_kw)
-        if show_beam and not self.pv:
-            self.add_beam(beam, beamcolor)
+        self._beam(show_beam, beamcolor, beampos, d)
 
     def add_rgb(self, xskip: int = 1, yskip: int = 1,
                 stretch: list[str, str, str] = ['linear'] * 3,
@@ -818,6 +850,7 @@ class PlotAstroData(AstroFrame):
                 stretchpower: float = 0,
                 show_beam: bool = True,
                 beamcolor: list[str, str, str] = ['red', 'green', 'blue'],
+                beampos: list[list[float, float] | None] = [None, None, None],
                 **kwargs) -> None:
         """Use PIL.Image and imshow of matplotlib. kwargs must include the arguments of AstroData to specify the data to be plotted. A three-element array ([red, green, blue]) is supposed for all arguments, except for xskip, yskip and show_beam, including vmax and vmin.
 
@@ -827,7 +860,8 @@ class PlotAstroData(AstroFrame):
             stretchscale (float, optional): color scale is asinh(data / stretchscale). Defaults to None.
             stretchpower (float, optional): color scale is ((data / vmin)**(1 - stretchpower) - 1) / (1 - stretchpower) / ln(10). 0 means the linear scale. 1 means the logarithmic scale. Defaults to 0.
             show_beam (bool, optional): Defaults to True.
-            beamcolor (str, optional): Matplotlib color. Defaults to 'gray'.
+            beamcolor (list, optional): Matplotlib color. Defaults to ['red', 'green', 'blue'].
+            beampos (list, optional): Relative position. Defaults to None.
         """
         from PIL import Image
 
@@ -860,9 +894,7 @@ class PlotAstroData(AstroFrame):
                     im.putpixel((i, j), value)
             axnow.imshow(im, extent=[x[0], x[-1], y[0], y[-1]])
             axnow.set_aspect(np.abs((x[-1]-x[0]) / (y[-1]-y[0])))
-        if show_beam and not self.pv:
-            for i in range(3):
-                self.add_beam(beam[i], beamcolor[i])
+        self._beam(show_beam, beamcolor, beampos, d)
 
     def set_axis(self, title: dict | str | None = None, **kwargs) -> None:
         """Use Axes.set_* of matplotlib. kwargs can include the arguments of PlotAxes2D to adjust x and y axis.
